#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import logging

from connexion import NoContent
from flask import session
from sqlalchemy.exc import SQLAlchemyError

from api.admin import is_admin
from db.group import Member, Group
from db.handler import db_session
from db.user import User


logger = logging.getLogger('api.user')


def get_user_with_groups(uid):
    """
    get user by id with groups
    :param uid: user id
    :return: user with groups
    """
    u = db_session.query(User)
    user = u.filter(User.id == uid).one_or_none()
    if not user:
        logger.warning("user with id {0} not found".format(uid))
        return None
    user = user.dump()
    user['groups'] = []
    for member in db_session.query(Member).filter(Member.user_id == uid).all():
        group = db_session.query(Group).filter(Group.id == member.group_id).one().dump()
        user['groups'].append(group)
    return user


def get_users():
    """
    get all users (admins)
    :return: list of user
    """
    if not is_admin():
        return NoContent, 401
    return [get_user_with_groups(u.id) for u in db_session.query(User).all() if u]


def add_user(u):
    """
    add new user
    :param u: user
    :return: user
    """
    if not is_admin():
        return NoContent, 401
    if 'admin' == u['dom_name']:
        logger.error("cannot add user admin, this name is reserved")
        return NoContent, 500
    u = User(**u)
    try:
        db_session.add(u)
        db_session.commit()
        db_session.refresh(u)
        return u.dump(), 201
    except SQLAlchemyError:
        logger.exception("error while creating account")
        return NoContent, 500


def remove_user(name):
    """
    remove existing user
    :param name: user
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    try:
        db_session.query(User).filter(User.dom_name == name).delete()
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while creating account")
        return NoContent, 500


def get_user(uid):
    """
    get user
    :param uid: user id
    :return: user (with groups)
    """
    if not is_admin():
        return NoContent, 401
    return get_user_with_groups(uid), 200


def find_user(name):
    """
    find user by name
    :param name: user name
    :return: user with groups
    """
    if is_admin():
        u = db_session.query(User).filter(User.dom_name == name).one_or_none()
        if not u:
            logger.warning("user with name {0} not found".format(name))
            return NoContent, 404
        return get_user_with_groups(u.id), 200
    if 'username' in session:
        user = db_session.query(User).filter(User.dom_name == session['username']).one_or_none()
        if not user:
            return NoContent, 401
        u = db_session.query(User).filter(User.dom_name == name).one_or_none()
        if not u:
            return NoContent, 401
        for admin in db_session.query(Member).filter(Member.user_id == user.id and Member.admin).all():
            if db_session.query(Member).filter(Member.group_id == admin.group_id and Member.user_id == u.id).one_or_none():
                return get_user_with_groups(u.id), 200
        return NoContent, 401
    return NoContent, 401


def get_myself():
    """
    get your own info
    :return: yourself and your groups
    """
    if 'username' not in session:
        return NoContent, 500
    u = db_session.query(User).filter(User.dom_name == session['username']).one_or_none()
    if not u:
        logger.error("session user {0} not in database".format(session['username']))
        return NoContent, 500
    return get_user_with_groups(u.id), 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import hashlib
import logging
import random

from connexion import NoContent
from flask import session
from sqlalchemy.exc import SQLAlchemyError

from db.group import Group, Member
from db.handler import db_session
from db.service import Service
from db.user import User

logger = logging.getLogger('api.admin')
allowed_chars = u'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'


def is_admin():
    """
    check if a user is an admin (excluding group admins)
    :return: admin yes/no
    """
    if 'admin' in session:
        return True
    else:
        if 'username' in session:
            u = db_session.query(User)
            user = u.filter(User.dom_name == session['username']).one_or_none()
            if user:
                group = db_session.query(Group).filter(Group.name == 'admins').one()
                if db_session.query(Member).filter(Member.group == group and Member.user == user).one_or_none():
                    return True
        return False


def user_is_group_admin(user, group):
    """
    check if a user is a group admin
    :param user: user name
    :param group: group name
    :return: admin yes/no
    """
    user = db_session.query(User).filter(User.dom_name == user).one_or_none()
    group = db_session.query(Group).filter(Group.name == group).one_or_none()
    if not user or not group:
        return False
    if db_session.query(Member).filter(Member.group_id == group.id and Member.user_id == user.id and Member.admin).one_or_none():
        return True
    return False


def is_group_admin(group):
    """
    inclusive search of admins and group admins
    :param group: name or id of the group to check, note that it's the database uid (- uid_init)
    :return: admin yes/no
    """
    if is_admin():
        return True
    if 'username' in session:
        user = db_session.query(User).filter(User.dom_name == session['username']).one_or_none()
        if not user:
            return False
        if isinstance(group, int):
            if db_session.query(Member).filter(Member.group_id == group and Member.user_id == user.id and Member.admin).one_or_none():
                return True
        else:
            return user_is_group_admin(user.dom_name, group)
    return False


def get_admins():
    """
    get a list of all users that are in the group admins
    :return: list of users
    """
    if not is_admin():
        return NoContent, 401
    group = db_session.query(Group).filter(Group.name == 'admins').one()
    users = []
    for group_user in db_session.query(Member).filter(Member.group == group).all():
        users.append(db_session.query(User).filter(User.id == group_user.user_id).one_or_none())
    return [u.dump() for u in users]


def add_admin(name):
    """
    add an existing user to admins
    :param name: dom_name
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    user = db_session.query(User).filter(User.dom_name == name).one_or_none()
    if not user:
        return NoContent, 404
    group = db_session.query(Group).filter(Group.name == 'admins').one()
    try:
        db_session.add(Member(group=group, user=user, admin=True))
        db_session.commit()
        return NoContent, 201
    except SQLAlchemyError:
        logger.exception("error while adding admin")
        return NoContent, 500


def remove_admin(name):
    """
    remove an existing user from admins
    :param name: dom_name
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    user = db_session.query(User).filter(User.dom_name == name).one_or_none()
    if not user:
        return NoContent, 404
    group = db_session.query(Group).filter(Group.name == 'admins').one()
    try:
        db_session.query(Member).filter(Member.group == group and Member.user == user).delete()
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while removing admin")
        return NoContent, 500


def get_services():
    """
    list all service accounts
    :return: list of dict(name, access)
    """
    if not is_admin():
        return NoContent, 401
    return [dict(name=s.name, access=s.access) for s in db_session.query(Service).all()], 200


def add_service(name):
    """
    add a service account
    :param name: name of the service
    :return: secret key in plain text, this will only be possible once!
    """
    if not is_admin():
        return NoContent, 401
    try:
        secret = ''.join(random.choice(allowed_chars) for c in range(24))
        s = Service(name=name,
                    access=''.join(random.choice(allowed_chars) for c in range(12)),
                    secret=hashlib.sha256(secret.encode('utf-8')).hexdigest())
        db_session.add(s)
        db_session.commit()
        db_session.refresh(s)
        service = s.dump()
        service['secret'] = secret
        return service, 201
    except SQLAlchemyError:
        logger.exception("error while adding service")
        return NoContent, 500


def remove_service(name):
    """
    remove a service account
    :param name: name of the service account
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    service = db_session.query(Service).filter(Service.name == name).one_or_none()
    if not service:
        return NoContent, 404
    try:
        db_session.delete(service)
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while removing admin")
        return NoContent, 500



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import logging

from connexion import NoContent
from flask import session
from sqlalchemy.exc import SQLAlchemyError

from api.admin import is_admin, is_group_admin
from db.group import Member, Group
from db.handler import db_session
from db.user import User

logger = logging.getLogger('api.group')


def get_groups(active=False):
    """
    list all groups (admins only)
    :param active: only show active groups
    :return: list of group
    """
    if not is_admin():
        return NoContent, 401
    if active:
        groups = [g.dump() for g in db_session.query(Group).all() if g.active]
    else:
        groups = [g.dump() for g in db_session.query(Group).all()]
    return groups, 200


def add_group(g):
    """
    add a group (admins only)
    :param g: group
    :return: group
    """
    if not is_admin():
        return NoContent, 401
    u = db_session.query(User).filter(User.dom_name == g['dom_name']).one_or_none()
    if not u:
        return "could not find user {0} for ownership".format(g['dom_name']), 404
    g.pop('dom_name', None)
    try:
        g = Group(**g, user_id=u.id)
        db_session.add(g)
        db_session.commit()
        db_session.refresh(g)
        return g.dump(), 201
    except SQLAlchemyError:
        logger.exception("error while creating group")
        return NoContent, 500


def update_group(gid, g):
    """
    update a group (admins and group admins)
    :param gid: group id
    :param g: group
    :return: success or fail
    """
    if not is_group_admin(gid):
        return NoContent, 401
    group = db_session.query(Group).filter(Group.id == gid).one_or_none()
    group.pop('id', None)
    try:
        for k in g:
            setattr(group, k, g[k])
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500


def get_group_users(gid):
    """
    get users of a group (admins, group admins and members)
    :param gid: group id
    :return: list of user
    """
    if not is_admin():
        u = None
        if 'username' in session:
            u = db_session.query(User).filter(User.dom_name == session['username']).one_or_none()
        if not u or not db_session.query(Member).filter(Member.group_id == gid and Member.user_id == u.id).one_or_none():
            logger.warning("user {0} not found as part of group".format(session['username']))
            return NoContent, 401
    users = []
    for group_user in db_session.query(Member).filter(Member.group_id == gid).all():
        gu = db_session.query(User).filter(User.id == group_user.user_id).one_or_none()
        if gu:
            users.append(dict(dom_name=gu.dom_name, full_name=gu.full_name, admin=group_user.admin))
    return users, 200


def add_group_user(gid, u, admin):
    """
    add user to group (admins and group admins)
    :param gid: group id
    :param u: dom_name
    :param admin: add as admin
    :return:
    """
    if not is_group_admin(gid):
        return NoContent, 401
    u = db_session.query(User).filter(User.dom_name == u).one_or_none()
    if not u:
        return 'User does not exist', 404
    group = db_session.query(Group).filter(Group.id == gid).one()
    try:
        db_session.add(Member(group=group, user=u, admin=admin))
        db_session.commit()
        return NoContent, 201
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500


def remove_group_user(gid, u):
    """
    remove user from group (admins and group admins)
    :param gid: group id
    :param u: dom_name
    :return: success or failure
    """
    if not is_group_admin(gid):
        return NoContent, 401
    u = db_session.query(User).filter(User.dom_name == u).one_or_none()
    if not u:
        return 'User does not exist', 404
    group = db_session.query(Group).filter(Group.id == gid).one()
    try:
        db_session.query(Member).filter(Member.group == group and Member.user == u).delete()
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import logging
import hashlib
import sys
from time import time

from connexion import NoContent
from flask import session
from flask_ldap3_login import LDAP3LoginManager, AuthenticationResponseStatus
from jose import JWTError, jwt, ExpiredSignatureError

from app import config
from db.handler import db_session
from db.service import Service
from db.user import User

logger = logging.getLogger('api.auth')
ldap_manager = None
tokens = {}


def init_ldap():
    if config.authentication().get('method') == 'ldap':
        auth_config = dict()
        auth_config['LDAP_HOST'] = config.authentication().get('host')
        auth_config['LDAP_PORT'] = int(config.authentication().get('port'))
        auth_config['LDAP_USE_SSL'] = bool(config.authentication().get('ssl'))
        auth_config['LDAP_BASE_DN'] = config.authentication().get('base_dn')
        auth_config['LDAP_USER_RDN_ATTR'] = config.authentication().get('rdn_attr')
        auth_config['LDAP_USER_LOGIN_ATTR'] = config.authentication().get('login_attr')
        auth_config['LDAP_BIND_USER_DN'] = config.authentication().get('bind_user')
        auth_config['LDAP_BIND_USER_PASSWORD'] = config.authentication().get('bind_pass')
        ldm = LDAP3LoginManager()
        ldm.init_config(auth_config)
        return ldm
    return None


def generate_token(identifier):
    """
    generate a JWT token
    :param identifier: name of user or service
    :return: JWT token
    """
    timestamp = int(time())
    payload = {
        "iss": config.token().get('issuer'),
        "iat": int(timestamp),
        "exp": int(timestamp + int(config.token().get('lifetime'))),
        "sub": str(identifier),
    }
    secret = config.token().get('secret')
    if not secret:
        secret = config.general().get('secret')
    return jwt.encode(payload, secret, algorithm=config.token().get('algorithm'))


def user_by_token(token, required_scopes=None):
    result = validate(token)
    if result:
        return dict(sub=next((name for name, t in tokens.items() if t == token), None), uid=result)
    return None


def validate(token):
    """
    validate if token is valid and not expired
    :param token: JWT token
    :return: user id, service id, or None if not valid or expired
    """
    if token not in tokens.values():
        logger.warning("invalid token supplied {0}".format(token))
        return None
    name = next((name for name, t in tokens.items() if t == token), None)
    if not name:
        return None
    logger.debug("token request for {0}".format(name))
    try:
        secret = config.token().get('secret')
        if not secret:
            secret = config.general().get('secret')
        jwt.decode(token, secret, algorithms=[config.token().get('algorithm')])
        if 'admin' in session:
            return session['admin']
        if 'service' in session:
            return session['service']
        user = db_session.query(User).filter(User.dom_name == name).one_or_none()
        if not user:
            return None
        return user.id
    except ExpiredSignatureError:
        logger.debug("token {0} for {1} expired, removing it".format(token, name))
        tokens.pop(name, None)
    except JWTError:
        logger.exception("error decoding token")
    return None


def access_secret_verify(access, secret):
    """
    validate access and secret for services
    :param access: access code
    :param secret: sha256 of secret
    :return: service name or None
    """
    try:
        s = db_session.query(Service)
        service = s.filter(Service.access == access).one_or_none()
        if not service:
            logger.warning("could not identify service by id {0}".format(access))
            return None
        if service.secret != hashlib.sha256(secret.encode('utf-8')).hexdigest():
            logger.error("verification failed for service with id {0}".format(access))
        return service.name, service.id
    except Exception as e:
        logger.error("failed login: {0}".format(e))
        return None


def login(username, password):
    """
    login user or service
    :param username: username or service
    :param password: password of sha256(secret)
    :return: token, 200 or 401
    """
    if 'username' in session:
        return "You are already logged in {0}".format(session['username']), 500
    if username == config.admin().get('access') and hashlib.sha256(config.admin().get('secret').encode('utf-8')).hexdigest() == password:
        session['username'] = 'admin'
        session['admin'] = sys.maxsize
        token = generate_token('admin')
        tokens['admin'] = token
        return token, 200
    (service, sid) = access_secret_verify(username, password)
    if service:
        session['username'] = service
        session['service'] = sid
        token = generate_token(service)
        tokens[service] = token
        return token, 200
    if not ldap_manager:
        init_ldap()
    if ldap_manager:
        if AuthenticationResponseStatus.success == ldap_manager.authenticate(username, password):
            session['username'] = username
            token = generate_token(username)
            tokens[username] = token
            return token, 200
    return NoContent, 401


def logout():
    """
    logout user or service
    :return: 200
    """
    if 'username' in session:
        tokens.pop(session['username'], None)
        del(session['username'])
    if 'admin' in session:
        tokens.pop(session['admin'], None)
        del(session['admin'])
    if 'service' in session:
        tokens.pop(session['service'], None)
        del(session['service'])
    if 'token' in session:
        del(session['token'])
    return NoContent, 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import logging

from connexion import NoContent
from dateutil import parser
from flask import session
from sqlalchemy.exc import SQLAlchemyError

from api.admin import is_admin, user_is_group_admin
from api.group import get_groups
from db.group import Group, Member
from db.handler import db_session
from db.resource import Resource, ResourceUsage
from db.user import User

logger = logging.getLogger('api.resource')


def get_resources():
    """
    list all resources (admins)
    :return: list of resource
    """
    if not is_admin():
        return NoContent, 401
    return [r.dump() for r in db_session.query(Resource).all()], 200


def add_resource(name):
    """
    add new resource (admins)
    :param name: resource name
    :return: resource
    """
    if not is_admin():
        return NoContent, 401
    try:
        r = Resource(name=name, active=True)
        db_session.add(r)
        db_session.commit()
        db_session.refresh(r)
        return r.dump(), 201
    except SQLAlchemyError:
        logger.exception("error while creating resource")
        return NoContent, 500


def get_resource_groups(rid):
    """
    get groups associated with resource (admins)
    :param rid: resource id
    :return: list of group
    """
    if not is_admin():
        return NoContent, 401

    def evaluate_resource_groups(resource, gid):
        for group in resource.groups:
            if gid == group.id:
                return True
        return False

    r = db_session.query(Resource).filter(Resource.id == rid).one_or_none()
    if not r:
        return NoContent, 404
    groups, code = get_groups(True)
    return [group for group in groups if evaluate_resource_groups(r, group['id'])], 200


def update_resource(rid, resource_update):
    """
    update existing resource (admins)
    :param rid: resource id
    :param resource_update: resource
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    r = db_session.query(Resource).filter(Resource.id == rid).one_or_none()
    if not r:
        return NoContent, 404
    try:
        for k in resource_update:
            setattr(r, k, resource_update[k])
        db_session.commit()
        return NoContent, 200
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500


def add_resource_group(rid, name):
    """
    associate group with resource (admins)
    :param rid: resource id
    :param name: group name
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    r = db_session.query(Resource).filter(Resource.id == rid).one_or_none()
    g = db_session.query(Group).filter(Group.name == name).one_or_none()
    if not r or not g:
        return NoContent, 404
    if g in r.groups:
        return 'Group already assigned to resource', 500
    try:
        r.groups.append(g)
        db_session.commit()
        return NoContent, 201
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500


def remove_resource_group(rid, name):
    """
    remove group association with resource
    :param rid: resource id
    :param name: group name
    :return: success or failure
    """
    if not is_admin():
        return NoContent, 401
    r = db_session.query(Resource).filter(Resource.id == rid).one_or_none()
    g = db_session.query(Group).filter(Group.name == name).one_or_none()
    if not r or not g:
        return NoContent, 404
    if g not in r.groups:
        return 'Group not assigned to resource', 500
    try:
        r.groups.remove(g)
        db_session.commit()
        return NoContent, 201
    except SQLAlchemyError:
        logger.exception("error while updating group")
        return NoContent, 500


def add_resource_usage(usages):
    """
    add resource usage records
    :param usages: list of ResourceUsage
    :return: success or failure
    """
    if not is_admin():
        if 'service' not in session:
            return NoContent, 401
    if not is_admin():
        invalid = [u for u in usages if u['r'] != session['username']]
        if len(invalid) > 0:
            return 'Invalid records found, can only insert your own', 500
        for usage in usages:
            usage['resource'] = usage['r']
            usage['user'] = usage['u']
            usage.pop('r', None)
            usage.pop('u', None)
            db_session.add(ResourceUsage(**usage))
        return NoContent, 201
    for usage in usages:
        usage['resource'] = usage['r']
        usage['user'] = usage['u']
        usage['start'] = parser.parse(usage['start'])
        usage['end'] = parser.parse(usage['end'])
        usage.pop('r', None)
        usage.pop('u', None)
        db_session.add(ResourceUsage(**usage))
    return NoContent, 201


def get_resource_usage(r, u=None, start=None, end=None):
    """
    get resource usage for a resource for a given user
    :param u: user
    :param r: resource
    :param start: start time
    :param end: end time
    :return: list of ResourceUsage
    """
    resource = db_session.query(Resource).filter(Resource.name == r).one_or_none()
    user = db_session.query(User).filter(User.dom_name == u).one_or_none()
    if not is_admin():
        if not resource or not user:
            return NoContent, 401
        allowed = False
        if u and session['username'] is not u:
            for g in resource.groups:
                if db_session.query(Member).filter(Member.group_id ==  g.id and Member.user_id == user.id).one_or_none():
                    if user_is_group_admin(u, g.name):
                        allowed = True
        if not allowed:
            return NoContent, 401
    if not resource or not user:
        return NoContent, 404
    usages = db_session.query(ResourceUsage).filter(ResourceUsage.resouce == r and
                                                    ResourceUsage.user == u and
                                                    ResourceUsage.start == start and
                                                    ResourceUsage.end == end).all()

    return [u.dump() for u in usages], 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import api.admin
import api.group
import api.user
import api.resource
import api.auth



#!/usr/bin/env python
# -*- coding: utf-8 -*-#
#
#
# Copyright (C) 2018 University of Zurich. All rights reserved.
#
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import os
import sys

from setuptools import setup

version = "0.5"

requirements = ["click",
                "click-log",
                "python-dateutil",
                "sqlalchemy",
                "connexion",
                "connexion[swagger-ui]",
                "flask-cors",
                "flask-ldap3-login",
                "python-jose",
                "psutil",
                "gevent"]

test_requirements = ["pytest", "tox"]

if sys.argv[-1] == "tag":
    os.system("git tag -a {0} -m 'version {1}'".format(version, version))
    os.system("git push origin master --tags")
    sys.exit()

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    os.system("python setup.py bdist_wheel upload")
    sys.exit()

if sys.argv[-1] == "test":
    try:
        modules = map(__import__, test_requirements)
    except ImportError as e:
        raise ImportError("{0} is not installed. Install your test requirements.".format(
            str(e).replace("No module named ", ""))
        )
    os.system('py.test')
    sys.exit()

setup(name="acpy",
      version=version,
      description="Accounting Center API",
      long_description=open("README.md").read(),
      author="Pim Witlox",
      author_email="pim.witlox@uzh.ch",
      url="https://github.com/uzh/acpy",
      license="GPLv3",
      entry_points={
          "console_scripts": [
              "acpy = app:cli",
          ]
      },
      packages=["api"],
      install_requires=requirements,
      python_requires=">=3.5",
      keywords="Web, Python, Python3, REST",
      project_urls={
          "Documentation": "https://acpy.readthedocs.io/en/latest/",
          "Source": "https://github.com/uzh/acpy/",
          "Tracker": "https://github.com/uzh/acpy/issues",
      },
      test_suite="tests",
      tests_require=test_requirements,
      classifiers=["Development Status :: 3 - Alpha",
                   "Intended Audience :: System Administrators",
                   "Natural Language :: English",
                   "Environment :: Console",
                   "License :: OSI Approved :: MIT License",
                   "Programming Language :: Python",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6",
                   "Topic :: Software Development :: Libraries",
                   "Topic :: Utilities"],
      )
